﻿using System.Windows;

// VMdemoWPF
// Example of an approach to ViewModel for scientific software, using WPF
// Released for educational purposes, without warranty or restrictions.
// Compliments of:
//    Michael A. Covington, Ph.D., Consultant
//    Covington Innovations
//    http://www.covingtoninnovations.com/michael

// Key idea:
//
// The purpose of the ViewModel paradigm is to restrict interaction between
// the user interface (MainWindow, the "view") and the computation engine 
// (the "model" in older nomenclature).  See the comments in MainWindowViewModel.cs.
//
// Note the following data bindings in MainWindow.xaml:
//
//   Main window:
//      Cursor="{Binding CursorBusyOrDefault}" 
//         Causes it to show the "busy" cursor when the computation is running.
//
//   "Seconds to count down" input textbox:
//      Text="{Binding SecondsToCount, UpdateSourceTrigger=PropertyChanged}"
//         Binds its content to the string property SecondsToCount, which is updated
//         whenever the user types anything; by default, textboxes only update
//         their data bindings when the user finishes typing and clicks on something else.
//      IsEnabled="{Binding NotRunning}"
//         Causes it to be enabled only when the computation is not running.
//      Cursor="{Binding CursorBusyOrDefault}"
//         As with main window.  Textboxes do not inherit the containing window's cursor.
//
//   Displayed countdown read-only textbox:
//      Text="{Binding DisplayedCountdown}" 
//         Binds its content to the string property DisplayedCountdown, which is where
//         the computation writes its output.
//      Cursor="{Binding CursorBusyOrDefault}"
//         As with main window.  Textboxes do not inherit the containing window's cursor.
//
//   Start button:
//      IsEnabled="{Binding Ready}"
//         Causes it to be enabled only when the computation is ready to start.
//
//   Cancel button:
//      Cursor="Arrow"
//         Causes the arrow cursor to be displayed at all times.  (This button is
//         used when the program is running and displaying the "busy" cursor on all
//         of its other components.)
//      IsEnabled="{Binding Running}"
//         Causes it to be enabled only when the computation is running.
//
//   Log textbox:
//      Cursor="{Binding CursorBusyOrDefault}"
//         As with main window.  Textboxes do not inherit the containing window's cursor.
//      Note that the log textbox has no data binding; the computation writes into it
//      using the MainWindowViewModel.AppendToLog method.

// See also the comments in MainWindowViewModel.cs and Computation.cs.

namespace VMdemoWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MainWindowViewModel vm;

        #region Main window constructor

        public MainWindow()
        {
            vm = new MainWindowViewModel(this);  // create a ViewModel object
            this.DataContext = vm; // make it this window's data context
            InitializeComponent();
        }

        #endregion

        #region GUI event handlers,  which call methods in the ViewModel

        private void buttonStart_Click(object sender, RoutedEventArgs e)
        {
            // Disable warning 4014 so compiler won't tell us to use "await".
            // Launching asynchronously is intentional here.
            #pragma warning disable 4014
            vm.LaunchComputation();
            #pragma warning restore 4014
        }

        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            vm.CancelComputation();
        }

        #endregion
    }
}
