﻿using System.Threading;
using System.Threading.Tasks;
using System.ComponentModel; 
using System.Runtime.CompilerServices; 
using System.Windows.Input; 

// VMdemoWPF
// Example of an approach to ViewModel for scientific software, using WPF
// Released for educational purposes, without warranty or restrictions.
// Compliments of:
//    Michael A. Covington, Ph.D., Consultant
//    Covington Innovations
//    http://www.covingtoninnovations.com/michael

// Key idea:
//
// The purpose of the ViewModel paradigm is to restrict interaction between
// the user interface (MainWindow, the "view") and the computation engine 
// (the "model" in older nomenclature).
//
// The MainWindowViewModel object contains everything the MainWindow and
// the computation engine need to communicate with each other.  They do
// not communicate with each other directly; they use it as an intermediary.
//
// Thus, the programmer can easily manage all possible interactions between
// the two.  The user interface code does not become cluttered with heterogeneous
// material interacting in various ways with the computation or with the GUI.
//
// If the user interface were to change completely (e.g., to WinForms or to
// a command line interface), MainWindowViewModel would have to be rewritten
// but the computation engine would not.  Conversely, major changes in the
// computation not involving a change in the user experience might involve
// changes to MainWindowViewModel but not to MainWindow.
//
// In a WPF application, communication between MainWindow and MainWindowViewModel
// takes place almost entirely through data binding.  This includes some unusual
// kinds of data, such as an object that is either the default cursor or the 
// "wait" cursor depending on the state of the program, and can be bound to the
// Cursor property of appropriate objects.

// See also the comments in MainWindow.xaml.cs and Computation.cs.


namespace VMdemoWPF
{
    /// <summary>
    /// ViewModel for MainWindow of this program. 
    /// If there were multiple windows, there would be a 
    /// ViewModel class for each, appropriately named.
    /// </summary>
    public class MainWindowViewModel: INotifyPropertyChanged
    {
        #region Implementation of INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        ///     Notifies listeners that a property value has changed.
        /// </summary>
        /// <param name="name">
        ///     Name of the property used to notify listeners.  
        ///     Usually omitted and supplied by CallerMemberName.
        /// </param>
        protected void OnPropertyChanged([CallerMemberName] string name = null)
        {
            PropertyChangedEventHandler eventHandler = this.PropertyChanged;
            if (eventHandler != null)
            {
                eventHandler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

        #region Constructor (initialization)

        private MainWindow mainWindow;

        private MainWindowViewModel() { }  // do not use argumentless constructor

        /// <summary>
        /// Create the ViewModel by passing it a reference to the MainWindow to
        /// which it belongs, so that it can access that window directly in
        /// situations where data binding isn't sufficient.
        /// </summary>
        public MainWindowViewModel(MainWindow win)
        {
            mainWindow = win;
            SecondsToCount = "5";
            DisplayedCountdown = "--";
            Running = false;
            Canceler = new CancellationTokenSource();
            // Initialized here so it will never be null,
            // but re-initialized before each use so it will be fresh
        }

        #endregion

        #region Canceler for signaling asynchronous processes to exit

        public CancellationTokenSource Canceler;

        #endregion

        #region Data displayed on the screen

        /// <summary>
        /// String holding the number of seconds to count (typed by the user).
        /// Bound to the textbox where it is typed in.
        /// </summary>
        public string SecondsToCount
        {
            get
            {
                return _SecondsToCount;
            }
            set
            {
                _SecondsToCount = value;
                OnPropertyChanged();
                OnPropertyChanged("Ready");  // this affects the Ready property too
            }
        }

        private string _SecondsToCount;

        /// <summary>
        /// String holding the displayed countdown.
        /// Bound to the textbox where it is displayed.
        /// </summary>
        public string DisplayedCountdown
        {
            get
            {
                return _DisplayedCountdown;
            }
            set
            {
                _DisplayedCountdown = value;
                OnPropertyChanged();
            }
        }

        private string _DisplayedCountdown;

        /// <summary>
        /// The second text box is a write-only log, like writing to a file,
        /// so instead of being data-bound to a string, it receives data through
        /// calls to this method.
        /// </summary>
        /// <param name="s">
        /// String to be written.  Be sure to include end-of-line mark if needed.
        /// </param>
        public void AppendToLog(string s)
        {
            // Any method in the ViewModel that directly manipulates the MainWindow
            // (not through binding) may be performing a cross-thread call.
            // Here's how it's done:

            mainWindow.textBoxLog.Dispatcher.InvokeAsync(
                () =>
                {
                    mainWindow.textBoxLog.AppendText(s);
                    mainWindow.textBoxLog.ScrollToEnd();
                }
            );
        }

        #endregion

        #region Program state indicators

        /// <summary>
        /// Read-only property indicating whether the computation is ready to start.
        /// True if Running is false and there is a valid integer in the SecondsToCount box.
        /// Bound to Start button's Enabled property.
        /// </summary>
        public bool Ready
        {
            get
            {
                int junk;
                return (Running == false && int.TryParse(SecondsToCount, out junk));
            }
            // There is no setter.
            // Setters of Running and SecondsToCount affect Ready and must
            // do OnPropertyChanged("Ready") as well as their own OnPropertyChanged.
        }

        /// <summary>
        /// Read-only property indicating whether the user interface can accept input.
        /// True if Running is false.
        /// Bound to Seconds To Count textbox's Enabled property.
        /// </summary>
        public bool NotRunning
        {
            get
            {
                return !Running;
            }
            // There is no setter.
            // Setter of Running affects NotRunning and must do
            // OnPropertyChanged("NotRunning") as well as its own OnPropertyChanged.
        }

        /// <summary>
        /// Property indicating whether the computation is currently in progress.
        /// Enabled property of the Cancel button is bound to this.
        /// Its setter also controls the cursor (arrow vs. wait cursor).
        /// </summary>
        public bool Running
        {
            get
            {
                return _Running;
            }
            set
            {
                _Running = value;
                OnPropertyChanged();
                OnPropertyChanged("Ready");  // this one also affects Ready
                OnPropertyChanged("NotRunning"); // and NotRunning
                OnPropertyChanged("CursorBusyOrDefault"); // and CursorBusyOrDefault
            }
        }

        private bool _Running;

        /// <summary>
        /// Read-only property.
        /// A WPF mouse cursor that is the "wait" cursor when the computation is running
        /// and is null (signifying default) otherwise.  The Cursor property of WPF elements
        /// can be bound to this for automatic cursor changes.
        /// </summary>
        public Cursor CursorBusyOrDefault
        {
            get
            {
                if (_Running)
                {
                    return Cursors.Wait;
                }
                else
                {
                    return null;
                }
            }
        }

        #endregion

        #region Methods to launch and cancel the computation

        /// <summary>
        /// Launch the computation in a concurrent process.
        /// Called by the event handler of the Start button.
        /// </summary>
        public async Task LaunchComputation()
        {
            Canceler = new CancellationTokenSource(); // fresh one for each run
            Running = true;
            await Task.Factory.StartNew( 
                () => Computation.Countdown(this) 
            );
            Running = false;
        }

        /// <summary>
        /// Set the CancellationTokenSource to indicate cancellation requested.
		/// (The process being cancelled needs to look at it periodically.)
        /// Called by the event handler of the Cancel button.
        /// </summary>
        public void CancelComputation()
        {
            Canceler.Cancel();
        }

        #endregion

    }
}
